@@ -1,4 +1,4 @@
-from pydantic import BaseModel, Field
+from pydantic import BaseModel, Field, root_validator
 from typing import Optional, List, Any, Dict
 from altscore.borrower_central.model.generics import GenericSyncResource, GenericAsyncResource, \
     GenericSyncModule, GenericAsyncModule
@@ -6,7 +6,8 @@ from altscore.borrower_central.model.generics import GenericSyncResource, Generi
 
 class DocumentsAPIDTO(BaseModel):
     id: str = Field(alias="id")
-    borrower_id: str = Field(alias="borrowerId")
+    borrower_id: Optional[str] = Field(alias="borrowerId", default=None)
+    deal_id: Optional[str] = Field(alias="dealId", default=None)
     key: str = Field(alias="key")
     label: Optional[str] = Field(alias="label", default=None)
     value: Optional[Any] = Field(alias="value")
@@ -15,6 +16,14 @@ class DocumentsAPIDTO(BaseModel):
     updated_at: Optional[str] = Field(alias="updatedAt")
     has_attachments: bool = Field(alias="hasAttachments")
 
+    @root_validator
+    def check_id_requirements(cls, values):
+        borrower_id = values.get("borrower_id")
+        deal_id = values.get("deal_id")
+        if not borrower_id and not deal_id:
+            raise ValueError("At least one of borrowerId or dealId must be provided")
+        return values
+
     class Config:
         populate_by_name = True
         allow_population_by_field_name = True
@@ -22,10 +31,19 @@ class DocumentsAPIDTO(BaseModel):
 
 
 class CreateDocumentDTO(BaseModel):
-    borrower_id: str = Field(alias="borrowerId")
+    borrower_id: Optional[str] = Field(alias="borrowerId", default=None)
+    deal_id: Optional[str] = Field(alias="dealId", default=None)
     key: str = Field(alias="key")
     value: Optional[Any] = Field(alias="value", default=None)
     tags: List[str] = Field(alias="tags", default=[])
+    
+    @root_validator(pre=True)
+    def check_id_requirements(cls, values):
+        borrower_id = values.get("borrowerId")
+        deal_id = values.get("dealId")
+        if not borrower_id and not deal_id:
+            raise ValueError("At least one of borrowerId or dealId must be provided")
+        return values
 
     class Config:
         populate_by_name = True
