@@ -7,8 +7,21 @@ from altscore.borrower_central.model.workflows import WorkflowExecutionResponseA
 from altscore.common.http_errors import raise_for_status_improved, retry_on_401, retry_on_401_async
 import httpx
 import datetime as dt
+from altscore.borrower_central.model.decisions import CurrentDecisionInExecution, PostDecisionToExecution
 from dateutil.parser import parse
 
+EXECUTION_NOTICE_SEVERITY_INFO = "info"
+EXECUTION_NOTICE_SEVERITY_ERROR = "error"
+EXECUTION_NOTICE_SEVERITY_DEBUG = "debug"
+
+class ExecutionNotice(BaseModel):
+    message: str = Field(alias="message")
+    severity: str = Field(alias="severity")
+
+    class Config:
+        populate_by_name = True
+        allow_population_by_field_name = True
+        allow_population_by_alias = True
 
 class CreateExecutionDTO(BaseModel):
     workflow_id: Optional[str] = Field(alias="workflowId", default=None)
@@ -16,6 +29,7 @@ class CreateExecutionDTO(BaseModel):
     workflow_version: Optional[str] = Field(alias="workflowVersion", default=None)
     workflow_type: Optional[str] = Field(alias="workflowType", default=None)
     borrower_id: Optional[str] = Field(alias="borrowerId", default=None)
+    deal_id: Optional[str] = Field(alias="dealId", default=None)
     billable_id: Optional[str] = Field(alias="billableId", default=None)
     batch_id: Optional[str] = Field(alias="batchId", default=None)
     tags: Optional[List[str]] = Field(alias="tags", default=[])
@@ -25,6 +39,16 @@ class CreateExecutionDTO(BaseModel):
         allow_population_by_field_name = True
         allow_population_by_alias = True
 
+
+class UpdateExecutionDTO(BaseModel):
+    unsuccessful_sources: Optional[bool] = Field(alias="unsuccessfulSources", default=None)
+
+    class Config:
+        populate_by_name = True
+        allow_population_by_field_name = True
+        allow_population_by_alias = True
+
+
 class ExecutionBatchMeta(BaseModel):
     id: str = Field(alias="id")
     item_index: int = Field(alias="itemIndex")
@@ -45,12 +69,31 @@ class ExecutionAPIDTO(BaseModel):
     execution_batch: Optional[ExecutionBatchMeta] = Field(alias="executionBatch", default=None)
     billable_id: Optional[str] = Field(alias="billableId")
     borrower_id: Optional[str] = Field(alias="borrowerId")
+    deal_id: Optional[str] = Field(alias="dealId", default=None)
+    current_decision: Optional[CurrentDecisionInExecution] = Field(alias="currentDecision", default=None)
     status: Optional[str] = Field(alias="status", default=None)
     tags: Optional[List[str]] = Field(alias="tags", default=[])
+    unsuccessful_sources: Optional[bool] = Field(alias="unsuccessfulSources", default=None)
     is_success: Optional[bool] = Field(alias="isSuccess", default=None)
     is_billable: Optional[bool] = Field(alias="isBillable", default=None)
     is_re_scoring: Optional[bool] = Field(alias="isReScoring", default=None)
     created_at: str = Field(alias="createdAt")
+    execution_time: Optional[int] = Field(alias="executionTime", default=None)
+    response_time: Optional[int] = Field(alias="responseTime", default=None)
+
+    class Config:
+        populate_by_name = True
+        allow_population_by_field_name = True
+        allow_population_by_alias = True
+
+
+class ExecutionInputDataAPIDTO(BaseModel):
+    id: Optional[str] = Field(alias="id")
+    workflow_id: str = Field(alias="workflowId")
+    workflow_alias: str = Field(alias="workflowAlias")
+    workflow_version: str = Field(alias="workflowVersion")
+    input: Dict = Field(alias="input")
+    created_at: str = Field(alias="createdAt")
 
     class Config:
         populate_by_name = True
@@ -62,6 +105,7 @@ class ExecutionOutputDataAPIDTO(BaseModel):
     id: Optional[str] = Field(alias="id")
     billable_id: Optional[str] = Field(alias="billableId")
     borrower_id: Optional[str] = Field(alias="borrowerId")
+    deal_id: Optional[str] = Field(alias="dealId", default=None)
     workflow_id: str = Field(alias="workflowId")
     workflow_alias: str = Field(alias="workflowAlias")
     workflow_version: str = Field(alias="workflowVersion")
@@ -88,6 +132,9 @@ class CreateExecutionOutput(BaseModel):
     error_message: Optional[str] = Field(alias="errorMessage", default=None)
     billable_id: Optional[str] = Field(alias="billableId", default=None)
     borrower_id: Optional[str] = Field(alias="borrowerId", default=None)
+    deal_id: Optional[str] = Field(alias="dealId", default=None)
+    execution_time: Optional[int] = Field(alias="executionTime", default=None)
+    notices: Optional[List[ExecutionNotice]] = Field(alias="notices", default=[])
 
     class Config:
         populate_by_name = True
@@ -103,6 +150,7 @@ EXECUTION_STATUS_COMPLETE = "complete"
 
 class ExecutionState(BaseModel):
     status: str = Field(alias="status")
+    execution_batch_id: Optional[str] = Field(alias="executionBatchId", default=None)
     callback_at: dt.datetime = Field(alias="callbackAt")
     state: Dict = Field(alias="state")
     updated_at: Optional[dt.datetime] = Field(alias="updatedAt", default=None)
@@ -116,6 +164,7 @@ class ExecutionState(BaseModel):
     def from_api_dto(cls, data: Dict):
         return cls(
             status=data["status"],
+            execution_batch_id=data["executionBatchId"],
             callback_at=parse(data["callbackAt"]),
             state=data["state"],
             updated_at=parse(data["updatedAt"]) if data["updatedAt"] else None
@@ -124,6 +173,7 @@ class ExecutionState(BaseModel):
     def to_api_dto(self):
         return {
             "status": self.status,
+            "executionBatchId": self.execution_batch_id,
             "callbackAt": self.callback_at.isoformat(),
             "state": self.state
         }
@@ -162,6 +212,7 @@ class ExecutionState(BaseModel):
 
 
 class ExecutionSync(GenericSyncResource):
+    input: ExecutionInputDataAPIDTO
     output: ExecutionOutputDataAPIDTO
     state: ExecutionState
 
@@ -171,12 +222,28 @@ class ExecutionSync(GenericSyncResource):
     def _state(self, resource_id):
         return f"{self.base_url}/v1/{self.resource}/{resource_id}/state"
 
+    def _input(self, resource_id):
+        return f"{self.base_url}/v1/{self.resource}/{resource_id}/input"
+
     def _output(self, resource_id):
         return f"{self.base_url}/v1/{self.resource}/{resource_id}/output"
 
     def _output_attachments(self, resource_id):
         return f"{self.base_url}/v1/{self.resource}/{resource_id}/output/attachments"
 
+    @retry_on_401
+    def get_input(self):
+        with httpx.Client() as client:
+            response = client.get(
+                self._input(self.data.id),
+                headers=self._header_builder(),
+                timeout=300
+            )
+            raise_for_status_improved(response)
+            self.input = ExecutionInputDataAPIDTO.parse_obj(response.json())
+        return self.input
+
+
     @retry_on_401
     def get_output(self):
         with httpx.Client() as client:
@@ -238,6 +305,28 @@ class ExecutionSync(GenericSyncResource):
             raise_for_status_improved(response)
             self.get_output()
 
+    @retry_on_401
+    def post_decision(self, decision_data: PostDecisionToExecution):
+        with httpx.Client() as client:
+            response = client.post(
+                f"{self.base_url}/v1/{self.resource}/{self.data.id}/decisions",
+                headers=self._header_builder(),
+                json=PostDecisionToExecution.parse_obj(decision_data).dict(by_alias=True, exclude_none=True),
+                timeout=300
+            )
+            raise_for_status_improved(response)
+
+    @retry_on_401
+    def delete_decision(self):
+        with httpx.Client() as client:
+            response = client.delete(
+                f"{self.base_url}/v1/{self.resource}/{self.data.id}/decisions",
+                headers=self._header_builder(),
+                timeout=300
+            )
+            raise_for_status_improved(response)
+
+
     @retry_on_401
     def retry(self, execution_mode: Optional[str] = None):
         headers = self._header_builder()
@@ -254,6 +343,7 @@ class ExecutionSync(GenericSyncResource):
 
 
 class ExecutionAsync(GenericAsyncResource):
+    input: ExecutionInputDataAPIDTO
     output: ExecutionOutputDataAPIDTO
     state: ExecutionState
 
@@ -263,12 +353,29 @@ class ExecutionAsync(GenericAsyncResource):
     def _state(self, resource_id):
         return f"{self.base_url}/v1/{self.resource}/{resource_id}/state"
 
+    def _input(self, resource_id):
+        return f"{self.base_url}/v1/{self.resource}/{resource_id}/input"
+
     def _output(self, resource_id):
         return f"{self.base_url}/v1/{self.resource}/{resource_id}/output"
 
     def _output_attachments(self, resource_id):
         return f"{self.base_url}/v1/{self.resource}/{resource_id}/output/attachments"
 
+
+    @retry_on_401_async
+    async def get_input(self):
+        async with httpx.AsyncClient() as client:
+            response = await client.get(
+                self._input(self.data.id),
+                headers=self._header_builder(),
+                timeout=300
+            )
+            raise_for_status_improved(response)
+            self.input = ExecutionInputDataAPIDTO.parse_obj(response.json())
+        return self.input
+
+
     @retry_on_401_async
     async def get_output(self):
         async with httpx.AsyncClient() as client:
@@ -330,6 +437,27 @@ class ExecutionAsync(GenericAsyncResource):
             raise_for_status_improved(response)
             await self.get_output()
 
+    @retry_on_401_async
+    async def post_decision(self, decision_data: Dict):
+        async with httpx.AsyncClient() as client:
+            response = await client.post(
+                f"{self.base_url}/v1/{self.resource}/{self.data.id}/decisions",
+                headers=self._header_builder(),
+                json=PostDecisionToExecution.parse_obj(decision_data).dict(by_alias=True, exclude_none=True),
+                timeout=300
+            )
+            raise_for_status_improved(response)
+
+    @retry_on_401_async
+    async def delete_decision(self):
+        async with httpx.AsyncClient() as client:
+            response = await client.delete(
+                f"{self.base_url}/v1/{self.resource}/{self.data.id}/decisions",
+                headers=self._header_builder(),
+                timeout=300
+            )
+            raise_for_status_improved(response)
+
     @retry_on_401_async
     async def retry(self, execution_mode: Optional[str] = None):
         headers = self._header_builder()
@@ -349,7 +477,7 @@ class ExecutionSyncModule(GenericSyncModule):
 
     def __init__(self, altscore_client):
         super().__init__(altscore_client, sync_resource=ExecutionSync, retrieve_data_model=ExecutionAPIDTO,
-                         create_data_model=CreateExecutionDTO, update_data_model=None, resource="executions")
+                         create_data_model=CreateExecutionDTO, update_data_model=UpdateExecutionDTO, resource="executions")
 
     @retry_on_401
     def query_outputs(self, **kwargs):
@@ -368,12 +496,33 @@ class ExecutionSyncModule(GenericSyncModule):
             raise_for_status_improved(response)
             return [ExecutionOutputDataAPIDTO.parse_obj(x) for x in response.json()]
 
+    @retry_on_401
+    def overwrite_principal(self, old_principal_id: str, new_principal_id: str, from_date: Optional[str] = None,
+                             to_date: Optional[str] = None):
+        payload = {
+            "oldPrincipalId": old_principal_id,
+            "newPrincipalId": new_principal_id
+        }
+        if from_date is not None:
+            payload["fromDate"] = from_date
+        if to_date is not None:
+            payload["toDate"] = to_date
+
+        with httpx.Client(base_url=self.altscore_client._borrower_central_base_url) as client:
+            response = client.post(
+                f"/v1/{self.resource}/commands/overwrite-principal",
+                json=payload,
+                headers=self.build_headers()
+            )
+            raise_for_status_improved(response)
+            return None
+
 
 class ExecutionAsyncModule(GenericAsyncModule):
 
     def __init__(self, altscore_client):
         super().__init__(altscore_client, async_resource=ExecutionAsync, retrieve_data_model=ExecutionAPIDTO,
-                         create_data_model=CreateExecutionDTO, update_data_model=None, resource="executions")
+                         create_data_model=CreateExecutionDTO, update_data_model=UpdateExecutionDTO, resource="executions")
 
     @retry_on_401_async
     async def query_outputs(self, **kwargs):
@@ -391,3 +540,20 @@ class ExecutionAsyncModule(GenericAsyncModule):
             )
             raise_for_status_improved(response)
             return [ExecutionOutputDataAPIDTO.parse_obj(x) for x in response.json()]
+
+    @retry_on_401_async
+    async def overwrite_principal(self, old_principal_id: str, new_principal_id: str, from_date: Optional[str] = None,
+                                 to_date: Optional[str] = None):
+        payload = {
+            "oldPrincipalId": old_principal_id,
+            "newPrincipalId": new_principal_id
+        }
+
+        async with httpx.AsyncClient(base_url=self.altscore_client._borrower_central_base_url) as client:
+            response = await client.post(
+                f"/v1/{self.resource}/commands/overwrite-principal",
+                json=payload,
+                headers=self.build_headers()
+            )
+            raise_for_status_improved(response)
+            return None
