@@ -15,6 +15,7 @@ import datetime as dt
 class ClientAPIDTO(BaseModel):
     id: str = Field(alias="clientId")
     partner_id: str = Field(alias="partnerId")
+    borrower_id: str = Field(alias="borrowerId", default=None)
     status: str = Field(alias="status")
     external_id: str = Field(alias="externalId")
     tax_id: str = Field(alias="taxId")
@@ -34,6 +35,7 @@ class ClientAPIDTO(BaseModel):
 
 class CreateClientDTO(BaseModel):
     partner_id: str = Field(alias="partnerId")
+    borrower_id: Optional[str] = Field(alias="borrowerId", default=None)
     external_id: str = Field(alias="externalId")
     legal_name: str = Field(alias="legalName")
     tax_id: str = Field(alias="taxId")
@@ -54,6 +56,7 @@ class UpdateClientDTO(BaseModel):
     legal_name: Optional[str] = Field(alias="legalName", default=None)
     email_address: Optional[str] = Field(alias="emailAddress", default=None)
     phone_number: Optional[str] = Field(alias="phoneNumber", default=None)
+    borrower_id: Optional[str] = Field(alias="borrowerId", default=None)
 
 
 class ClientBase:
@@ -63,6 +66,13 @@ class ClientBase:
             client_id: str, product_family: str
     ) -> (str, Optional[dict]):
         return f"/v2/clients/{client_id}/credit-accounts/{product_family}"
+    
+
+    @staticmethod
+    def _reservations(
+            client_id: str, product_family: str
+    ) -> (str, Optional[dict]):
+        return f"/v2/clients/{client_id}/credit-accounts/{product_family}/reservations"
 
     @staticmethod
     def _status(client_id: str):
@@ -130,6 +140,17 @@ class ClientAsync(ClientBase):
                 renew_token=self.renew_token,
                 data=CreditAccountAPIDTO.parse_obj(response.json())
             )
+        
+    @retry_on_401_async
+    async def create_reservation(self, product_family: str, input: dict) -> None:
+        async with httpx.AsyncClient(base_url=self.base_url) as client:
+            response = await client.post(
+                self._reservations(self.data.id, product_family),
+                json=input,
+                headers=self._header_builder(partner_id=self.data.partner_id),
+                timeout=30
+            )
+            raise_for_status_improved(response)
 
     @retry_on_401_async
     async def enable(self):
@@ -309,6 +330,17 @@ class ClientSync(ClientBase):
                 renew_token=self.renew_token,
                 data=CreditAccountAPIDTO.parse_obj(response.json())
             )
+    
+    @retry_on_401
+    def create_reservation(self, product_family: str, input: dict) -> None:
+        with httpx.Client(base_url=self.base_url) as client:
+            response = client.post(
+                self._reservations(self.data.id, product_family),
+                json=input,
+                headers=self._header_builder(partner_id=self.data.partner_id),
+                timeout=30
+            )
+            raise_for_status_improved(response)
 
     @retry_on_401
     def enable(self):
@@ -518,7 +550,7 @@ class ClientsAsyncModule(GenericAsyncModule):
             response = await client.post(
                 "/v2/clients",
                 headers=headers,
-                json=CreateClientDTO.parse_obj(new_entity_data).dict(by_alias=True),
+                json=CreateClientDTO.parse_obj(new_entity_data).dict(by_alias=True, exclude_none=True),
                 timeout=30
             )
             raise_for_status_improved(response)
@@ -577,7 +609,7 @@ class ClientsSyncModule(GenericSyncModule):
             response = client.post(
                 "/v2/clients",
                 headers=headers,
-                json=CreateClientDTO.parse_obj(new_entity_data).dict(by_alias=True),
+                json=CreateClientDTO.parse_obj(new_entity_data).dict(by_alias=True, exclude_none=True),
                 timeout=30
             )
             raise_for_status_improved(response)
